/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:io';
import 'package:build/src/builder/build_step.dart';
import 'dart:async';
import 'package:source_gen/source_gen.dart';

class SaverGenerator extends Generator {
  @override
  FutureOr<String?> generate(LibraryReader library, BuildStep buildStep) async {
    final assetId = await buildStep.resolver.assetIdForElement(library.element);
    final currentPath = "${Directory.current.path}/${assetId.path}";
    final currentFile = File(currentPath);
    String currentFileContent = await currentFile.readAsString();
    final RegExp regex = RegExp(r"\/\/\/\#\#\#(.+?)\#\#\#");
    final String functionIdentifier =
        regex.firstMatch(currentFileContent)?.group(1) ?? '';

    await File(currentPath.replaceAll(
            RegExp(r'\d+\.wt'), functionIdentifier + ".welltested_test"))
        .writeAsString(currentFileContent.replaceAll(regex, ''));
    await currentFile.delete();

    int startIndex = currentPath.indexOf(".wt");
    int endIndex = currentPath.indexOf(".dart");

    String currentMockFilePath =
        currentPath.replaceRange(startIndex + 3, endIndex, ".mocks");
    if (await File(currentMockFilePath).exists()) {
      final currentMockFileContent =
          await File(currentMockFilePath).readAsString();
      await File(currentMockFilePath.replaceAll(
              RegExp(r'\d+\.wt'), functionIdentifier + ".welltested_test"))
          .writeAsString(currentMockFileContent);
      await File(currentMockFilePath).delete();
    }

    return super.generate(library, buildStep);
  }
}
