/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:build/src/builder/build_step.dart';
import 'package:source_gen/source_gen.dart';
import 'package:welltested/annotation.dart';
import 'package:welltested/src/repositories/dart_server_repository.dart';
import 'package:welltested/src/executors/fixer_executor.dart';
import 'package:welltested/src/repositories/path_repository.dart';
import 'package:welltested/src/utilities/process.dart';

class WelltestedGenerator extends GeneratorForAnnotation<Welltested> {
  @override
  Future<FutureOr<String>> generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) async {
    if (element is! ClassElement) {
      final String friendlyName = element.displayName;
      throw InvalidGenerationSourceError(
        'Generator cannot target `$friendlyName`.',
        todo: 'Remove the [Welltested] annotation from `$friendlyName`.',
      );
    }
    final dartServerExecutor = DartServerRepository();
    await dartServerExecutor.initialize();

    final PathRepository pathExecutor = PathRepository();
    await pathExecutor.init(buildStep);

    final fixerExecutor = FixerExecutor(dartServerExecutor.server);
    return generateWelltestedSource(buildStep, element,
        annotation: annotation,
        fixerExecutor: fixerExecutor,
        dartServerRepository: dartServerExecutor,
        pathRepository: pathExecutor);
  }

  @override
  FutureOr<String> generate(LibraryReader library, BuildStep buildStep) {
    return super.generate(library, buildStep);
  }
}
