/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';
import 'package:welltested/src/extensions/class_element_extension.dart';

class ModelsExecutor {
  Future<List> processModels(
      List<ClassElement> classElements, BuildStep buildStep) async {
    final List<ClassElement> complexModels = [];
    final List<ClassElement> simpleModels = [];

    for (final element in classElements) {
      if (!element.isSimple) {
        complexModels.add(element);
        continue;
      }
      final nonSDKElements = element.nonDartElements;
      if (nonSDKElements.isEmpty) {
        simpleModels.add(element);
        continue;
      }

      bool isComplexModel = false;
      for (final nonSDKElement in nonSDKElements) {
        final moreNonSDKElements = nonSDKElement.nonDartElements;
        if (moreNonSDKElements.isNotEmpty) {
          isComplexModel = true;
          break;
        }
      }
      if (isComplexModel) {
        complexModels.add(element);
      } else {
        //TODO: Maybe define a max field limit length of comined models
        simpleModels.add(element);
        simpleModels.addAll(nonSDKElements);
      }
    }
    final List<String> simpleModelCode = [];

    for (final simpleModel in simpleModels) {
      simpleModelCode.add(await simpleModel.simplefiedSource(buildStep));
    }

    return [simpleModelCode, complexModels];
  }
}
