/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:build/build.dart';
import 'package:welltested/src/executors/fixer_executor.dart';
import 'package:welltested/src/repositories/dart_server_repository.dart';
import 'package:welltested/src/repositories/path_repository.dart';
import 'package:welltested/src/repositories/writer_repository.dart';

class WriterExecutor {
  final PathRepository pathRepository;
  final WriterRepository writerRepository;
  final DartServerRepository dartServerRepository;
  final FixerExecutor fixerExecutor;

  WriterExecutor(this.pathRepository, this.writerRepository,
      this.dartServerRepository, this.fixerExecutor);

  Future<void> processGeneration(
      Uri fileImportPath, String content, String methodKey, int i) async {
    final file = await writerRepository.writeTestToCorrespondingFile(
        fileImportPath, content, methodKey);
    await dartServerRepository.addFilesToRoot([file.absolute.path]);
    final updatedContent =
        await fixerExecutor.findAndFixIssues(file.path, methodKey);
    await outputGeneration(i, methodKey, updatedContent);
    await file.delete();
    log.info(" Test Generated for ${methodKey}");
  }

  Future<void> outputGeneration(int i, String methodKey, String content) async {
    final asset = AssetId(pathRepository.inputLibraryInfo.package,
        pathRepository.correspondingTemporaryTestFilePath(i));
    print("output asset path ${asset.path}");
    await writerRepository.outputGeneratedFile(asset, content, methodKey);
  }
}
