/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:async';
import 'dart:convert';
import 'dart:developer';

import 'package:dotenv/dotenv.dart';
import 'package:http/http.dart' as http;
import 'package:welltested/src/models/test_request_data.dart';
import 'package:welltested/src/models/widget_test_request_data.dart';

class GenerationRepository {
  final baseUrl = 'https://api.welltested.ai/v1';

  Future<Map<String, dynamic>> buildUnitTests(
      TestRequestData testRequestData) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => "");
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final response = await http.post(Uri.parse('$baseUrl/ai/generatetests/'),
          body: testRequestData.toJson(), headers: {"api_key": apiKey});
      return jsonDecode(response.body);
    } catch (e) {
      throw (e);
    }
  }

  Future<Map<String, dynamic>> buildWidgetTests(
      WidgetTestRequestData widgetTestRequestData) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => "");
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final response = await http.post(
          Uri.parse('$baseUrl/ai/generatetests/widget/'),
          body: widgetTestRequestData.toJson(),
          headers: {"api_key": apiKey});
      return jsonDecode(response.body);
    } catch (e) {
      throw (e);
    }
  }

  Future<Map<String, dynamic>> updateTest(
      {required String hashID, String? fixedTest, String? userTest}) async {
    final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
    final apiKey = env.getOrElse("WELLTESTED_API", () => "");
    if (apiKey.isEmpty) {
      throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
    }
    Map<String, dynamic> updatedData = {
      "hash_id": hashID,
    };
    if (fixedTest != null) updatedData['fixed_completion'] = fixedTest;
    if (userTest != null) updatedData['user_completion'] = userTest;
    final response = await http.post(Uri.parse('$baseUrl/ai/updatetest'),
        body: updatedData, headers: {"api_key": apiKey});

    return jsonDecode(response.body);
  }

  Future<Map<String, dynamic>> updateFixedTest(
      TestRequestData testRequestData, String fixedTest) async {
    log(jsonEncode(testRequestData.toJson()));
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => "");
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final response = await http.post(
          Uri.parse('$baseUrl/ai/updatefixedtests/'),
          body: testRequestData.toJson()
            ..addEntries([MapEntry("fixed_test_case", fixedTest)]),
          headers: {"api_key": apiKey});

      return jsonDecode(response.body);
    } catch (e) {
      throw (e);
    }
  }

  Future<Map<String, dynamic>> updateFixedTestWidget(
      WidgetTestRequestData testRequestData,
      String fixedTest,
      String testType) async {
    log(jsonEncode(testRequestData.toJson()));
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => "");
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final response = await http.post(
          Uri.parse('$baseUrl/ai/updatefixedtests/widget/'),
          body: testRequestData.toJson()
            ..addEntries([
              MapEntry("fixed_test_case", fixedTest),
              MapEntry('test_type', testType)
            ]),
          headers: {"api_key": apiKey});

      return jsonDecode(response.body);
    } catch (e) {
      throw (e);
    }
  }
}
