
/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:io';
import 'dart:isolate';
import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';

class PathRepository {
  late final String packageName;
  late final LibraryElement inputLibrary;
  late final AssetId inputLibraryInfo;
  Future<void> init(BuildStep buildStep) async {
    final package = await Isolate.packageConfig;
    packageName = package!.pathSegments[package.pathSegments.length - 3];
    inputLibrary = await buildStep.inputLibrary;
    inputLibraryInfo = await buildStep.resolver.assetIdForElement(inputLibrary);
  }

  String get relativeTestDirectory {
    return inputLibraryInfo.path
        .replaceAll("lib", "test")
        .replaceAll(".dart", '');
  }

  String correspondingTemporaryTestFilePath(int i) {
    return "${relativeTestDirectory}/$i.wt.dart";
  }

  Future<File?> correspondingPermanentTestFile(String functionName) async {
    final path =
        "${Directory.current.path}/${relativeTestDirectory}/$functionName.welltested_test.dart";
    return (await File(path).exists()) ? File(path) : null;
  }

  Future<File?> fileExistsAtPath(String path) async {
    return (await File(path).exists()) ? File(path) : null;
  }
}
