/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:collection';

import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';
import 'package:path/path.dart' as p;

Future<Map<Element, String>> extractImportFilesForElements(
    List<Element> elements, BuildStep buildStep) async {
  final entryLib = await buildStep.inputLibrary;
  var entryAssetId = await buildStep.resolver.assetIdForElement(entryLib);
  final typeUris = <Element, String>{};
  for (final element in elements) {
    final elementLibrary = element.library!;
    if (elementLibrary.isInSdk && !elementLibrary.name.startsWith('dart._')) {
      if (elementLibrary.isDartAsync || elementLibrary.isDartCore) {
        continue;
      }
      // For public SDK libraries, just use the source URI.
      typeUris[element] = elementLibrary.source.uri.toString();
      continue;
    }
    final exportingLibrary = _findExportOf([entryLib], element);

    try {
      final typeAssetId =
          await buildStep.resolver.assetIdForElement(exportingLibrary);

      if (typeAssetId.path.startsWith('lib/')) {
        typeUris[element] = typeAssetId.uri.toString();
      } else {
        typeUris[element] =
            p.relative(typeAssetId.path, from: p.dirname(entryAssetId.path));
      }
    } on UnresolvableAssetException {
      // Asset may be in a summary.
      typeUris[element] = exportingLibrary.source.uri.toString();
    }
  }

  return typeUris;
}

/// Returns a library which exports [element], selecting from the imports of
/// [inputLibraries] (and all exported libraries).
///
/// If [element] is not exported by any libraries in this set, then
/// [element]'s declaring library is returned.
LibraryElement _findExportOf(
    Iterable<LibraryElement> inputLibraries, Element element) {
  final elementName = element.name;
  if (elementName == null) {
    return element.library!;
  }

  final libraries = Queue.of([
    for (final library in inputLibraries) ...library.importedLibraries,
  ]);

  for (final library in libraries) {
    if (library.exportNamespace.get(elementName) == element) {
      return library;
    }
  }
  return element.library!;
}
