/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:io';

import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';

/// Read [https://github.com/dart-lang/build/issues/2689]('InconsistentAnalysisException: Requested result might be inconsistent with previously returned results')
Future<AstNode> getResolvedAstNodeForElement(
    Element element, BuildStep buildStep) async {
  final astNode = await buildStep.resolver.astNodeFor(element, resolve: true);
  if (astNode == null) throw ("Node of this element is null");
  return astNode;
}

Future<File> createTestFileInDirectory(
  String fullFilePath,
  String content,
  String methodInTest,
) async {
  final outputFile = File("${Directory.current.path}/test$fullFilePath"
      .replaceFirst(".dart", "_${methodInTest.toLowerCase()}_test.dart"));
  if (!await outputFile.exists()) {
    await outputFile.create(recursive: true);
  }
  return await outputFile.writeAsString(content);
}

Future<File?> getTestFileFromDirectory(
  String fullFilePath,
  String methodInTest,
) async {
  final testFile = File("${Directory.current.path}/test$fullFilePath"
      .replaceFirst(".dart", "_${methodInTest.toLowerCase()}_test.dart"));
  if (await testFile.exists()) {
    return testFile;
  } else {
    return null;
  }
}

bool isPrivate(String name) {
  return name.startsWith("_");
}
